# -*- coding:utf-8 -*-

# V1形式のデータベース（プレーンテキストファイル）からPostgreSQLに変換する。

V1DATA_DIR = '/home/hori/wwwdata/account'
CONF_FILE = '/home/hori/public_html/ac/conf/ac-conf.rb'

require 'rubygems'
gem 'activerecord'
require 'active_record'

require CONF_FILE
require './v1models/ac-items.rb' # DBAccessor
require './v1models/journal_entry.rb'
require APP_ROOT + '/models/user'
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/user_setting'
require APP_ROOT + '/models/journal_entry'
require APP_ROOT + '/models/budget'
require APP_ROOT + '/models/budget_detail'


def find_by_v1code(u, code)
  raise TypeError if !u.is_a?(User)
  r = AccountTitle.find(:first, :conditions => ['user_id = ? AND sort = ?', u.id, code])
  raise "notfound: uid = #{u.id}, login = #{u.login}, v1code = #{code}" if !r
  return r
end

# 1ユーザ分をコピーする
def copy_userdata(v1db, uid)
  # ユーザの生成
  passwd = v1db.get_passwd(uid)
  u = User.new(:login => passwd.name,
               :name => passwd.gecos,
               :email => passwd.mail,
               :expire => (passwd.expire ? passwd.expire : nil))
  u.crypted_password = passwd.passwd
  u.save!

  # 勘定科目の生成
  ac_titles = AcTitleStore.get(uid, v1db)
  ac_titles.each {|e|
    t = AccountTitle.new(:user => u,
                         :name => e.name,
                         :name_short => e.name_short,
                         :sort => e.code,
                         :ac_type => e.ac_type,
                         :cf_type => e.cf_type,
                         :description => e.desc,
                         :active => 1)
    t.save!
  }

  # ユーザ設定の生成
  prof = v1db.get_profile(uid)
  setting = UserSetting.new(
        :user => u,
        :unappropriated_item => find_by_v1code(u, prof.unappropriated_item).id,
        :opening_date => prof.opening_date,
        :fixed_date => prof.fixed,
        :closing_day => prof.closing_day,
        :ledger_last_item => (prof.ledger_last_item.to_i != 0 ? find_by_v1code(u, prof.ledger_last_item).id : nil),
        :theme_name => prof.theme_name,
        :entry_no_counter => 1)

  # 開始残高の移送
  StrStore.new(v1db.file_path.opening_balance(uid)).transaction_ro {|opdb|
    entry_no = (setting.entry_no_counter += 1)
    opdb.keys.each {|code|
      item = find_by_v1code(u, code)
      j = JournalEntry.new(:user => u,
                           :date => setting.opening_date,
                           :entry_no => entry_no,
                           :dr_code => item.id,
                           :cr_code => setting.unappropriated_item,
                           :amount => opdb[code].to_i,
                           :remarks => '',
                           :comment => '初期設定')
      j.save!
    }
  }
  setting.save!

  # 仕訳
  JournalStore.new(uid, v1db).transaction_ro {|store|
    store.each {|line|
      j = JournalEntry.new(:user => u,
                           :date => line.date,
                           :entry_no => line.num,
                           :dr_code => find_by_v1code(u, line.dr_code).id,
                           :cr_code => find_by_v1code(u, line.cr_code).id,
                           :amount => line.amount,
                           :remarks => line.remarks,
                           :comment => line.comment)
      j.save!
    }
  }

  # 予算
  v1db.budget_list(uid).each {|begin_date, end_date|
    budstore = StrStore.new(v1db.file_path.budget(uid, begin_date, end_date))
    budstore.transaction_ro {
      codes = budstore.keys
      break if codes.size <= 0
      b = Budget.new(:user => u,
                     :date => begin_date,
                     :comment => "")
      b.save!
      codes.each {|code|
        bd = BudgetDetail.new(:budget => b,
                              :account_title => find_by_v1code(u, code),
                              :amount => budstore[code])
        bd.save!
      }
    }
  }
end
                              
def migrate
  ActiveRecord::Base.establish_connection AR_OPTION

  User.transaction {
    v1db = DBAccessor.new(V1DATA_DIR)
    v1db.get_users().each {|uid|
      copy_userdata(v1db, uid)
    }
  }
end

migrate()
