# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require 'v1models/account_title.rb'
require 'v1models/profile.rb'
require 'v1models/passwd.rb'

class DBAccessor
  class FilePath
    def initialize(data_dir)
      @data_dir = data_dir
    end

    def lockfile()
      @data_dir + "/lockfile"
    end
  
    def passwd()
      @data_dir + "/_passwd"
    end

    def ac_titles(user)
      DBAccessor.valid_userid(user)
      @data_dir + "/" + user + "/titles"
    end

    def profile(user)
      DBAccessor.valid_userid(user)
      @data_dir + "/" + user + "/profile"
    end

  def journal_counter(user)
    DBAccessor.valid_userid(user)
    @data_dir + "/" + user + "/journal.counter"
  end

  def seq_counter(user)
    DBAccessor.valid_userid(user)
    @data_dir + "/" + user + "/seq.counter"
  end

  def journal(user)
    DBAccessor.valid_userid(user)
    @data_dir + "/" + user + "/journal"
  end

  def tmp_journal(user)
    DBAccessor.valid_userid(user)
    @data_dir + "/" + user + "/tmp-journal"
  end

  def subsidiary(userid)
    DBAccessor.valid_userid(userid)
    @data_dir + "/" + userid + "/subsidiary"
  end
  
  # 初期残高 = prof'opening-date'現在の残高。この翌日から仕訳を入力できる。
  def opening_balance(user)
    DBAccessor.valid_userid(user)
    @data_dir + "/" + user + "/opening"
  end

  def budget_dir(user)
    DBAccessor.valid_userid(user)
    @data_dir + "/#{user}/budget"
  end

  def budget(user, begin_date, end_date)
    raise TypeError, "begin_date not Date" if !begin_date.instance_of?(Date)
    raise TypeError, "end_date not Date"  if !end_date.instance_of?(Date)
    budget_dir(user) + "/#{begin_date}.#{end_date}"
  end

  def partner(user)
    DBAccessor.valid_userid(user)
    @data_dir + '/' + user + '/partner'
  end

    def partner_serial(user)
      DBAccessor.valid_userid(user)
      @data_dir + '/' + user + '/partner.counter'
    end
  end # of class FilePath

  # 予算期間の配列を返す
  def budget_list(user)
    ary = Array.new
    Dir.glob(@file_path.budget_dir(user) + "/????-??-??.????-??-??") {|fn|
      bdate, edate = File.basename(fn).split('.')
      ary << [Date.parse(bdate), Date.parse(edate)]
    }
    return ary
  end

  def self.valid_userid(userid)
    if !userid.is_a?(String)
      raise TypeError, "userid must be a String but was '#{userid.inspect}'"
    end
    raise SecurityError if userid =~ /[^a-zA-Z0-9]/
    userid.untaint if !userid.frozen?
  end

  def initialize(data_dir)
    @file_path = FilePath.new(data_dir)
    @passwd_store = PasswdStore.new(@file_path.passwd)
  end
  attr_reader :file_path

  # 複数のテーブルを更新するときに、このメソッドのブロックで囲む。
  def transaction()
    raise "nested transaction" if @transaction
    @rdonly = false
    @abort = false
    begin
      @transaction = true
      value = nil
      File.open(@file_path.lockfile(), "w") {|fp|
        fp.flock(File::LOCK_EX)
        begin
          catch(:dba_abort_transaction) do
            value = yield(self)
          end
        rescue Exception
          @abort = true
          raise
        end
      }
    ensure
      @transaction = false
    end
    value
  end

  # 複数のテーブルからデータを読んでいて、かつ更新されては困るときに、このメソッドの
  # ブロックで囲む。
  def transaction_ro()
    raise "nested transaction" if @transaction
    @rdonly = true
    @abort = false
    begin
      @transaction = true
      value = nil
      File.open(@file_path.lockfile(), "w") {|fp|
        fp.flock(File::LOCK_SH)
        catch(:dba_abort_transaction) do
          value = yield(self)
        end
      }
    ensure
      @transaction = false
    end
    value
  end

  def get_profile(userid)
    DBAccessor.valid_userid(userid)

    prof = Profile.new
    StrStore.new(@file_path.profile(userid)).transaction_ro {|profdb|
      prof.unappropriated_item = profdb.key?("unappropriated-item") ? 
                                       profdb["unappropriated-item"].to_i : nil
      prof.opening_date = profdb.key?("opening-date") && profdb["opening-date"] != "" ?
                                       Date.parse(profdb["opening-date"]) : nil
      prof.fixed = profdb.key?("fixed") && profdb["fixed"] != "" ?
                                              Date.parse(profdb["fixed"]) : nil
      prof.closing_day = profdb.key?("closing-day") ? profdb["closing-day"].to_i : nil
      prof.ledger_last_item = profdb.key?("ledger-last-item") ?
                                profdb["ledger-last-item"].to_i : nil
      prof.theme_name = profdb.key?("theme-name") ?
                                profdb["theme-name"] : nil
    }
    return prof
  end

  def set_profile(prof, userid)
    DBAccessor.valid_userid(userid)

    StrStore.new(@file_path.profile(userid)).transaction {|profdb|
      profdb["unappropriated-item"] = prof.unappropriated_item.to_s
      profdb["opening-date"] = prof.opening_date.to_s
      profdb["fixed"] = prof.fixed.to_s
      profdb["closing-day"] = prof.closing_day.to_s
      profdb["ledger-last-item"] = prof.ledger_last_item.to_s
      profdb["theme-name"] = (prof.theme_name || '')
    }
  end

  # 締め日の翌日から次の締め日まで
  def get_ac_month(userid)
    DBAccessor.valid_userid(userid)

    prof = get_profile(userid)
    begin_date = prof.fixed + 1
    closing_day = prof.closing_day

    if begin_date.day <= closing_day
      y = begin_date.year
      m = begin_date.mon
      d = closing_day
    else
      y = next_month(begin_date).year
      m = next_month(begin_date).mon
      d = closing_day
    end
    while !Date.exist?(y, m, d)
      d -= 1
    end
    end_date = Date.new(y, m, d)
    return [begin_date, end_date]
  end

  # 仕訳追加可能期間
  def get_addable_period(userid)
    DBAccessor.valid_userid(userid)

    fixed = get_profile(userid).fixed
    return [fixed + 1, (fixed + 1) + 50]
  end

  def user_exist?(userid)
    DBAccessor.valid_userid(userid)
    return @passwd_store.user_exist?(userid)
  end

  def add_user(ent)
    DBAccessor.valid_userid(ent.name)
    @passwd_store.add_user(ent)
  end

  def update_passwd(ent)
    @passwd_store.update_passwd(ent)
  end

  def get_users()
    @passwd_store.users()
  end

  def get_passwd(userid)
    DBAccessor.valid_userid(userid)
    return @passwd_store.get_passwd(userid)
  end

  def delete_user(name)
    return @passwd_store.delete_user(name)
  end
  
  def set_passwd(ent, userid)
    DBAccessor.valid_userid(userid)
    raise ArgumentError if ent.name != userid
    @passwd_store.update_passwd(ent)
  end

  # 仕訳、補助簿データを得る
  def get_subsidiary(item, begin_date, end_date, userid)
    raise TypeError if !item.is_a?(AcTitle)
    DBAccessor.valid_userid(userid)

    j_ary = []
    s_ary = []
    JournalStore.new(userid, self).transaction_ro {|store|
      store.each {|r|
        if r.date >= begin_date && r.date <= end_date &&
                (r.dr_code == item.code || r.cr_code == item.code)
          j_ary << r
        end
      }
      StrStore.new(@file_path.subsidiary(userid)).transaction_ro {|subdb|
        subdb.keys.each {|k|
          r = SubsidiaryRecord.parse(subdb[k])
          day = store[r.seq].date
          if day >= begin_date && day <= end_date && r.item == item.code
            s_ary << r
          end
        }
      }
    }
    return [j_ary, s_ary]
  end
end
