#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "pcview.rb"
require APP_ROOT + '/models/budget_detail'
require APP_ROOT + '/models/journal_entry'

class Form < PCView
  def lookup_amount(ary, item)
    ary.each {|r|
      return r.amo.to_i if r.account_title_id == item.id
    }
    return nil
  end

  # 指定された期間の実績を得る。
  def get_actual(begin_date, end_date)
    records = JournalEntry.find(:all,
                       :conditions => ['user_id = ? and date between ? and ?', 
                                       user.id, begin_date, end_date])
    result = {}
    records.each {|line|
      result[line.dr_code] = (result[line.dr_code] || 0) + line.amount
      result[line.cr_code] = (result[line.cr_code] || 0) - line.amount
    }
    return result
  end

  # [科目コード, 予算金額] の配列を得る。
  def get_budget(begin_date, end_date)
    budget = BudgetDetail.find_by_sql([<<EOF, user.id, begin_date, end_date])
SELECT B.account_title_id, sum(B.amount) as amo
    FROM budgets JOIN budget_details B ON B.budget_id = budgets.id
        JOIN account_titles ON B.account_title_id = account_titles.id
    WHERE budgets.user_id = ? and budgets.date between ? and ?
    GROUP BY B.account_title_id, account_titles.ac_type, account_titles.sort
    ORDER BY account_titles.ac_type, account_titles.sort
EOF
    return budget
  end

  def main
    begin_date = Date.new(cgi.get('byear').to_i, cgi.get('bmon').to_i, 
                          cgi.get('bday').to_i)
    end_date = Date.new(cgi.get('eyear').to_i, cgi.get('emon').to_i, 
                        cgi.get('eday').to_i)

    op_date = user.user_setting.opening_date
    begin_date = op_date + 1 if begin_date <= op_date
    end_date = op_date + 1 if end_date <= op_date

    budget = get_budget(begin_date, end_date)
    actual = get_actual(begin_date, end_date)

    # activeでないものもリストアップする
    ac_titles = AccountTitle.find(:all,
                                  :conditions => ['user_id = ?', user.id],
                                  :order => 'ac_type, sort')
    
    model = {
      :period => "自：#{begin_date} 至：#{end_date}",
      :group => []
    }

    gr = nil
    ac_titles.each {|item|
      if !gr || item.ac_type != gr[:ac_type]
        model[:group] << gr if gr && gr[:list].size > 0 # 前のグループ

        gr = {
          :ac_type => item.ac_type,
          :grname => AccountTitle::ACTYPE_NAMES[item.ac_type],
          :list => []
        }
      end

      b_amount = lookup_amount(budget, item)
      a_amount = actual[item.id]
      if b_amount || a_amount
        line = {
          :acname => item.name,
          :amounts => [(b_amount ? decimal_format(b_amount) : '-'), 
                       (a_amount ? decimal_format(a_amount * item.sign) : '-'), 
                       decimal_format(a_amount.to_i * item.sign - b_amount.to_i)]
        }
        if b_amount
          line[:amounts] << (Float(a_amount.to_i * item.sign) / Float(b_amount) * 100 + 0.5).to_i
        else
          line[:amounts] << '-'
        end
        gr[:list] << line
      end
    }
    model[:group] << gr if gr && gr[:list].size > 0

    page_out("actual_view.html", model)
  end
end

run(Form.new)
