# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "pcview.rb"
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/journal_entry'
require LIBRARY_DIR + '/datesup.rb'

class Journal_CommonAF < Wafu::ActionForm
  private
  def check_and_update(r, error, result, period)
    if r[1] == 0 && result.last then r[1] = result.last[1] end
    if r[2] == 0 && result.last then r[2] = result.last[2] end
    date = get_full_day(r[1], r[2], period[0])
    if !date
      error << "#{result.size + 1}行目の日付が正しくありません。"
    elsif date < period[0]
      error << "#{result.size + 1}行目の日付が締め日より前です。締め日より古いデータは登録できません。"
    elsif date > period[1]
      error << "#{result.size + 1}行目は、登録可能期間の外の日付です。メニューから「締め処理」を行って、古いデータを確定してください。"
    end

    r[3] = result.last[3] if r[3] == 0 && result.last
    r[4] = result.last[4] if r[4] == 0 && result.last
    if r[3] == r[4]
      error << "#{result.size + 1}行目の借方科目と貸方科目が同じになっています。"
    end
  end
end
                
class JournalActionForm < Journal_CommonAF
  def validate()
    error = []
    result = []

    period = UserSetting.get_addable_period(@app.user)

    lines = @app.cgi["lines"].first.to_i
    (1..lines).each {|i|
      r = []
      r[1] = @app.cgi["mon#{i}"].first.to_i
      r[2] = @app.cgi["day#{i}"].first.to_i
      r[3] = @app.cgi["drcode-#{i}"].first.to_i
      r[4] = @app.cgi["crcode-#{i}"].first.to_i
      r[5] = @app.cgi["amount-#{i}"].first.to_i
      r[6] = @app.cgi["remarks-#{i}"].first.strip
      r[7] = @app.cgi["comment-#{i}"].first.strip
      r[8] = @app.cgi["cont-#{i}"].first ? "on" : nil

      if r[3] != 0 || r[4] != 0 || r[5] != 0  # 1行目は常に有効な行
        check_and_update(r, error, result, period)
        result << r
      end
    }

    @journal_data = result
    @redirect = @app.cgi.get("redirect")
    return set_error(error)
  end
  attr_reader :journal_data
  attr_reader :redirect
end

class ConfirmAction < Wafu::Action
  private
  def execute_sub(af)
    result = af.journal_data

    # 表示
    model = {
      :action_url => @action_url,
      :redirect => @action_url == "cash.rb" ? @action_url : "journal.rb", # TODO:
      :list => [],
      :lines => result.length
    }
    result.each_with_index {|r, i|
      h = {
        :mon => "mon#{i + 1}",
        :mv => r[1],
        :day => "day#{i + 1}",
        :dv => r[2],
        :dr_sel => get_sel_active_items(@app.user, "drcode-#{i + 1}", r[3]),
        :cr_sel => get_sel_active_items(@app.user, "crcode-#{i + 1}", r[4]),
        :amount => "amount-#{i + 1}",
        :av => r[5],
        :remarks => "remarks-#{i + 1}",
        :rv => r[6],
        :comment => "comment-#{i + 1}",
        :cv => r[7]
      }
      if i != 0
        h[:cont] = {
          :cont => "cont-#{i + 1}", :contv => r[8],
          :ck => r[8] ? "checked" : nil
        }
      end
      model[:list] << h
    }

    if af.error?
      model[:errors] = af.errors
    end

    @app.page_out("jour2.html", model)
    return nil
  end
end
                
class DetermineAction < Wafu::Action
  def execute(af)
    period = UserSetting.get_addable_period(@app.user)
    prof = @app.user.user_setting

    JournalEntry.transaction {
      af.journal_data.each_with_index {|line, i|
        next if line[5] == 0

        # line = [nil, mon, day, drcode, crcode, amount, remarks, comment, cont]
        r = JournalEntry.new(:user => @app.user,
              :date => get_full_day(line[1], line[2], period[0]),
              :entry_no => (line[8] && add_data.last) ? add_data.last.num :
                           (prof.entry_no_counter += 1),
              :dr_code => line[3],
              :cr_code => line[4],
              :amount => line[5],
              :remarks => line[6],
              :comment => line[7])
        raise if r.date < period[0] || r.date > period[1] || r.dr_code == r.cr_code
        r.save!
      }
      prof.save!
    }

    # 次の伝票入力画面を表示
    @app.set_flash("notice", "notice: 伝票を登録しました。")
    @app.set_redirect(AppConfig[:site_top_url] + af.redirect)
    return nil
  end
end
