#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 仕訳の編集、削除
# 締め日前のデータは編集できない。

require 'pcview.rb'
require APP_ROOT + '/models/journal_entry'

class EditAF < Wafu::ActionForm
  def validate()
    error = []
    @seq = @app.cgi["seq"].first.to_i
    if @seq == 0
      error << "internal error: seqが正しくありません。"
      return set_error(error)
    end
    
    t = JournalEntry.find(@seq, :conditions => ['user_id = ?', @app.user.id])
    @mon = t.date.mon
    @day = t.date.day

    @ary = []
    entries = JournalEntry.find(:all, 
                       :conditions => ['user_id = ? and date = ? and entry_no = ?', 
                                       @app.user.id, t.date, t.entry_no])
    entries.each {|r|
      @ary << [r.dr_code, r.cr_code, r.amount, r.remarks, r.comment]
    }
    @lines = @ary.size + 2 # magic

    return set_error(error)
  end
  attr_reader :mon, :day, :seq, :ary
  attr_reader :lines
end

class EditForm < Wafu::Action
  def execute(af)
    period = UserSetting.get_addable_period(@app.user)

    model = {
      :period => "#{period[0]} 〜 #{period[1]}",
      :sel_date => get_sel_date("b", nil, af.mon, af.day),
      :seq => af.seq,
      :list => []
    }
    if af.error?
      model[:error] = {:message => af.errors}
    end

    for i in 1..af.lines do
      if i <= af.ary.size
        r = af.ary[i - 1]
        model[:list] << {
          :dr_sel => i == 1 ? get_sel_active_items(@app.user, "drcode-#{i}", r[0]) :
                    get_sel_active_items_with_zero(@app.user, "drcode-#{i}", r[0]),
          :cr_sel => i == 1 ? get_sel_active_items(@app.user, "crcode-#{i}", r[1]) :
                    get_sel_active_items_with_zero(@app.user, "crcode-#{i}", r[1]),
        :amount => "amount-#{i}", :av => r[2],
        :remarks => "remarks-#{i}", :rv => r[3],
        :comment => "comment-#{i}", :cv => r[4]
        }
      else
        model[:list] << {
        :dr_sel => get_sel_active_items_with_zero(@app.user, "drcode-#{i}", 0),
        :cr_sel => get_sel_active_items_with_zero(@app.user, "crcode-#{i}", 0),
        :amount => "amount-#{i}",
        :remarks => "remarks-#{i}",
        :comment => "comment-#{i}", }
      end
    end
    model[:lines] = af.lines
    @app.page_out('journal-edit.html', model)
  end
end

class DoneAF < Wafu::ActionForm
  def validate()
    errors = []
    result = []

    period = UserSetting.get_addable_period(@app.user)

    @mon = @app.cgi["bmon"].first.to_i
    @day = @app.cgi["bday"].first.to_i
    @seq = @app.cgi.get("seq").to_i

    new_day = get_full_day(@mon, @day, period[0])
    if !new_day
      errors << "日付が正しくありません。"
    else 
      if new_day < period[0]
        errors << "日付が締め日より前です。締め日より古いデータは登録できません。"
      elsif new_day > period[1]
        errors << "日付が登録可能期間の外です。メニューから「締め処理」を行って、古いデータを確定してください。"
      end
    end

    @lines = @app.cgi["lines"].first.to_i
    @ary = []
    (1..@lines).each {|i|
      r = [
        @app.cgi["drcode-#{i}"].first.to_i,
        @app.cgi["crcode-#{i}"].first.to_i,
        @app.cgi["amount-#{i}"].first.to_i,
        @app.cgi["remarks-#{i}"].first.strip,
        @app.cgi["comment-#{i}"].first.strip
      ]
      if r[2] != 0 && r[0] != 0 && r[0] == r[1]
        errors << "#{i}行目の借方科目と貸方科目が同じです。"
      end
      @ary << r
    }
    return set_error(errors)
  end
  attr_reader :mon, :day, :seq, :ary
  attr_reader :lines
end

class DoneForm < Wafu::Action
  def execute(af)
    JournalEntry.transaction {
      begin_day, end_day = UserSetting.get_addable_period(@app.user)

      t = JournalEntry.find(af.seq, :conditions => ['user_id = ?', @app.user.id])
      raise "internal error" if t.date < begin_day
      JournalEntry.delete_all(['user_id = ? and date = ? and entry_no = ?',
                               @app.user.id, t.date, t.entry_no])
      
      new_day = get_full_day(af.mon, af.day, begin_day)
      entry_no = (@app.user.user_setting.entry_no_counter += 1)
      prev = nil
      af.ary.each {|line|
        if line[2] != 0
          r = JournalEntry.new(:user => @app.user,
                               :date => new_day,
                               :entry_no => entry_no,
                               :dr_code => line[0] != 0 ? line[0] : prev.dr_code,
                               :cr_code => line[1] != 0 ? line[1] : prev.cr_code,
                               :amount => line[2],
                               :remarks => line[3],
                               :comment => line[4])
          raise "internal error" if r.dr_code == r.cr_code
          r.save!
          prev = r
        end
      }
      @app.user.user_setting.save!
    } # of transaction
    @app.page_out('journal-edit-done.html', {})
  end
end

class EditApp < PCView
  def main
    actions = {
      'edit' => [EditAF, EditForm, nil],
      'determine' => [DoneAF, DoneForm, EditForm]
    }
    dispatch(actions)
  end
end

EditApp.new.run()
