#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 月次比較表；週次比較表とほとんど同じつくり

require "pcview.rb"
require "comparison.rb"

class MonthStart < Wafu::Action
  def execute(af)
    closing_day = @app.user.user_setting.closing_day
    today = Date.today

    model = {
      :ey => today.year,
      :em => today.mon,
      :ed => closing_day.to_s
    }
    @app.page_out("month.html", model)
  end
end

class MonthStartReload < Wafu::Action
  def execute(af)
    model = {
      :ey => af.ey,
      :em => af.em,
      :ed => af.ed
    }
    if af.error?
      model[:error] = {:item => af.errors}
    end
    @app.page_out("month.html", model)
  end
end

class MonthAF < Wafu::ActionForm
  attr_reader :ey, :em, :ed

  def setup()
    @ey = @app.cgi.get("ey")
    @em = @app.cgi.get("em")
    @ed = @app.cgi.get("ed")
  end

  def validate()
    err = []
    err << "年または月が正しくありません。" if !Date.exist?(@ey.to_i, @em.to_i, 1)
    err << "日が正しくありません。" if @ed.to_i < 1 || @ed.to_i > 31
    return set_error(err)
  end
end

class MonthOut < Wafu::Action
  include ComparativeChart

  def get_prev_end(last_end, cur_end)
    raise TypeError if !cur_end.is_a?(Date) && !cur_end.is_a?(NilClass)

    if !cur_end
      y, m, d = last_end
    else
      t = cur_end << 1
      y = t.year; m = t.mon; d = last_end[2]
    end
    while d > 0 && !Date.exist?(y, m, d)
      d -= 1
    end
    return Date.new(y, m, d)
  end

  def execute(af)
    begin
      table_out("月次推移表", [af.ey.to_i, af.em.to_i, af.ed.to_i])
    rescue RangeError => e
      af.set_error([e.message])
      return MonthStartReload
    end
  end
end

class Form < PCView
  def main()
    actions = {
      "do" => [MonthAF, MonthOut, MonthStartReload],
      "*" => [nil, MonthStart, nil]
    }
    dispatch(actions)
  end
end

if __FILE__ == $0
  run(Form.new)
end
