# -*- mode:ruby; coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "ac-common.rb"
require "banner.rb"
require "amrita/template"
require LIBRARY_DIR + '/datesup'

class PCView < Wafu::WebApp
  def initialize(opt_hash = {})
    ActiveRecord::Base.establish_connection AR_OPTION

    opt = {
      "session_type" => "auth_required",
      "session_option" => {"tmpdir" => SESSION_DIR, "holdtime" => SESSION_HOLD_TIME},
      "passwd_store" => User
    }
    opt.update(opt_hash)
    super(opt)
  end

  def setup
    @user = User.find_by_login(@user) if @user
  end

  def internal_error_message_out()
    print <<EOF
Content-Type:text/html; charset=UTF-8

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
            "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
  <link rel="stylesheet" href="./default.css" type="text/css" charset="Shift_JIS">
  <title>内部エラー発生</title>
</head>
<body>
</table></ul></dl>
<h1>内部エラー発生</h1>
<p>予期しないエラーが発生しました。ご迷惑をお掛けします。
<p>プログラム修正の参考のため、エラーが発生したときの状況などをお送りいただけると助かります。よろしくお願いします。
#{comment_form()}
EOF
    print "<p>", CGI.escapeHTML($!.inspect), "\n"
    print "<p>"
    $@.each {|x| print CGI.escapeHTML(x), "<br>\n"}
    print "<hr>\n", CGI.escapeHTML(cgi.params.inspect)
    print <<EOF
<hr>
</body>
</html>
EOF
  end

  def login_form(mes)
    cgi_header_out(nil)

    print <<EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
            "http://www.w3.org/TR/html4/loose.dtd">
<html lang="ja">
<head>
  <link rel="stylesheet" href="./default.css" type="text/css" charset="Shift_JIS">
  <link rel="stylesheet" href="#{theme_url(user)}" type="text/css">
  <script charset="Shift_JIS" type="text/javascript" src="./scripts.js"></script>
  <title>ログイン: #{SITE_NAME}</title>
</head>
<body>
<h1>ログイン</h1>
EOF

    if mes == Wafu::LOGON_RETRY
      print "<p>ユーザIDまたはパスワードが違います。\n"
    elsif mes == Wafu::LOGON_EXPIRE
      print "<p>セッション維持時間を越えました。再ログインが必要です。\n"
    end
    print <<EOF
<form action="./login.rb" method="post">
<table class="frm">
  <tr><th nowrap align="right">ユーザID：<td><input type="text" name="user" size="10">
  <tr><th nowrap align="right">パスワード：<td><input type="password" name="pwd" size="10">
  <tr><td colspan="2" align="center">
    <input type="submit" value="ログイン">
</table>
</form>
<p>ケータイ版は、ログインしてから [ケータイにサイトのURLを送る] を行ってください。
<p><a href="sendpass.rb">[ユーザID・パスワードを忘れた場合]</a><br>
  <a href="signup.rb">[新規ユーザ登録]</a>

<p><a href="#{AppConfig[:site_top_url]}">[#{AppConfig[:site_name]} トップページ]</a>
EOF

    footer_out(nil)
  end

  def get_navbar_modal()
    r = <<EOF
<div class="adminmenu">
  <span class="adminmenu"><a href="logoff.rb">[ログアウト]</a></span>
</div>
EOF
    return r
  end

  def get_navigation_bar()
    page_id = File.basename((ENV['SCRIPT_FILENAME'] || "accountbook"), ".rb").gsub(/\./, '')
    r = <<EOF
<div class="adminmenu">
  <span class="adminmenu"><a href="./login.rb">[トップメニュー]</a></span>
  <span class="adminmenu"><a href="journal.rb">[振替伝票]</a></span>
  <span class="adminmenu"><a href="ledger.rb">[元帳]</a></span>
  <span class="adminmenu"><a href="jour.rb">[仕訳帳]</a></span>
  <span class="adminmenu"><a href="pl.rb">[財務諸表]</a></span>
  <span class="adminmenu"><a href="week.rb">[週次比較表]</a></span>
  <span class="adminmenu"><a href="month.rb">[月次比較表]</a></span><br>
  <span class="adminmenu"><a href="budget.rb">[計画設定]</a></span>
  <span class="adminmenu"><a href="actual.rb">[計画実績対比]</a></span>
  <span class="adminmenu"><a href="logoff.rb">[ログアウト]</a></span>
</div>
EOF
    return r
  end

  def cgi_header_out(dummy__ = nil)
    print cgi.header({"type" => "text/html", "charset" => "UTF-8"})
  end

  # for backward compatibility
  def header_out(cgi, title)
    cgi_header_out(cgi)

    print <<EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
            "http://www.w3.org/TR/html4/loose.dtd">
<html lang="ja">
<head>
  <link rel="stylesheet" href="./default.css" type="text/css" charset="Shift_JIS">
  <link rel="stylesheet" href="#{theme_url(user)}" type="text/css">
  <script charset="Shift_JIS" type="text/javascript" src="./scripts.js"></script>
  <title>#{title}: #{SITE_NAME}</title>
</head>
<body>
EOF
    print get_navigation_bar()
    print <<EOF
<h1>#{title}</h1>
EOF
  end

  def footer_out(dummy__ = nil)
    print get_footer()
    print <<EOF
</body>
</html>
EOF
  end

  def comment_form()
    key = CGI.escapeHTML(ENV['SCRIPT_FILENAME'] || "page-unknown")
    r = <<EOF
<form method="post" action="./feedback.rb">
  <input type="hidden" name="k" value="#{key}">
<table border="0" class="frm" style="margin:0 0 0 0">
EOF
    if !@user
      r += <<EOF
<tr><th align="right" nowrap>お名前：<td align="left"><input type="text" name="u" size="30">
<tr><th align="right">メールアドレス：<td align="left"><input type="text" name="m" size="30">
EOF
    end
    r += <<EOF
  <tr><th align="right" valign="top" nowrap>コメント：
    <td><textarea name="body" rows="6" cols="50"></textarea>
  <tr><td colspan="2" align="center">
    <input type="submit" value="送信">
</table>
</form>
EOF
    return r
  end

  def banner_text()
    page_id = File.basename((ENV['SCRIPT_FILENAME'] || "accountbook"), ".rb").gsub(/\./, '')
    return <<EOF
<br>
<!-- VC active -->
<SCRIPT type="text/javascript" LANGUAGE="JavaScript">
<!--
// ValueParameters
ValueHost = "hs0014065";
ValueLoaded = false;
ValueID = "pagebuster";
ValueVersion = "1.1";
ValueBannerType = "std";

ValueWidth = 468;
ValueHeight = 60;
//-->
</SCRIPT>
<SCRIPT type="text/javascript" LANGUAGE="Javascript" SRC="http://oz.valueclick.ne.jp/jsmaster"></SCRIPT>
<SCRIPT type="text/javascript" LANGUAGE="JavaScript">
<!--
if (ValueLoaded) ValueShowAd();
//-->
</SCRIPT>
<NOSCRIPT>
<A HREF="http://oz.valueclick.ne.jp/redirect?host=hs0014065&amp;size=468x60&amp;t=std&amp;b=#{page_id}&amp;noscript=1&amp;v=0" TARGET="_top">
<IMG BORDER="0" WIDTH="468" HEIGHT="60" ALT="Click here to visit our sponsor" SRC="http://oz.valueclick.ne.jp/cycle?host=hs0014065&amp;size=468x60&amp;t=std&amp;b=#{page_id}&amp;noscript=1">
  </A>
</NOSCRIPT>
<!-- vc active -->







<!-- VC active -->
<SCRIPT type="text/javascript" LANGUAGE="JavaScript">
<!--
// ValueParameters
ValueHost = "hs0014065";
ValueLoaded = false;
ValueID = "pagebuster";
ValueVersion = "1.1";
ValueBannerType = "std";

ValueWidth = 125;
ValueHeight = 125;
//-->
</SCRIPT>
<SCRIPT type="text/javascript" LANGUAGE="Javascript" SRC="http://oz.valueclick.ne.jp/jsmaster"></SCRIPT>
<SCRIPT type="text/javascript" LANGUAGE="JavaScript">
<!--
if (ValueLoaded) ValueShowAd();
//-->
</SCRIPT>

<NOSCRIPT>
<A HREF="http://oz.valueclick.ne.jp/redirect?host=hs0014065&amp;size=125x125&amp;t=std&amp;b=#{page_id}&amp;noscript=1&amp;v=0" TARGET="_top">
<IMG BORDER="0" WIDTH="125" HEIGHT="125" ALT="Click here to visit our sponsor" SRC="http://oz.valueclick.ne.jp/cycle?host=hs0014065&amp;size=125x125&amp;t=std&amp;b=#{page_id}&amp;noscript=1">
  </A>
</NOSCRIPT>
<!-- vc active -->
EOF
  end

  def get_footer()
    s = <<EOF
<hr noshade>
<div align="right">
Powered by <a href="http://www.nslabs.jp/accountbook.rhtml">Q's Account Book</a>.<br>
Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
</div>
EOF
    #'
    return s + banner_text()
  end

  def theme_url(user)
    prof = user ? user.user_setting : nil
    if prof && prof.theme_name && prof.theme_name != ''
      return AppConfig[:theme_base_url] + "/" + prof.theme_name + "/" + prof.theme_name + ".css"
    else
      return AppConfig[:theme_base_url] + "/" + AppConfig[:theme_default] + "/" + AppConfig[:theme_default] + ".css"
    end
end

  def page_out(filename, model)
    raise SecurityError if filename =~ /\//
                                      
    tmpl = Amrita::TemplateText.new(File.read("#{APP_ROOT}/templates/" + filename).untaint)
    tmpl.amrita_id = "tid"
    mapping = model.dup.update({
      :theme => theme_url(user),
      :navbar => Amrita::SanitizedString[get_navigation_bar()],
      :navbar_modal => Amrita::SanitizedString[get_navbar_modal()],
      :footer => Amrita::SanitizedString[get_footer()],
      :sitename => CGI.escapeHTML(SITE_NAME),
    })
    cgi_header_out()
    utext = ""
    tmpl.expand(utext, mapping)
    print utext
  end
end

class DateRangeActionForm < Wafu::ActionForm
  attr_reader :begin_date, :end_date
  attr_reader :by, :bm, :bd, :ey, :em, :ed

  def setup()
    @by = @app.cgi["byear"].first
    @bm = @app.cgi["bmon"].first
    @bd = @app.cgi["bday"].first
    @ey = @app.cgi["eyear"].first
    @em = @app.cgi["emon"].first
    @ed = @app.cgi["eday"].first
  end

  def validate()
    error = []
    jdb = Date.exist?(@by.to_i, @bm.to_i, @bd.to_i)
    jde = Date.exist?(@ey.to_i, @em.to_i, @ed.to_i)
    error << "開始日が正しくありません。" if (!jdb)
    error << "終了日が正しくありません。" if (!jde)
    if jdb && jde
      error << "終了日が開始日より古い日付になっています。" if (jde < jdb)
      @begin_date = Date.new1(jdb)
      @end_date = Date.new1(jde)
    end

    return set_error(error)
  end
end
                                  
def mod_render(file, model)
  raise SecurityError if file =~ /\//
  tmpl = Amrita::TemplateText.new(File.read("#{APP_ROOT}/templates/" + file).untaint)
  tmpl.amrita_id = "tid"
  utext = ""
  tmpl.expand(utext, model)
  return utext
end

# 日付選択コントロール
def get_sel_date(name_prefix, y, m, d)
  r = ""
  if y
    r << %Q(<input type="text" class="num" size="5" name="#{name_prefix}year" value="#{y}">年)
  end
  if m
    r << %Q(<input type="text" class="num" size="3" name="#{name_prefix}mon" value="#{m}">月)
  end
  if d
    r << %Q(<input type="text" class="num" size="3" name="#{name_prefix}day" value="#{d}">日)
  end
  r << %q!<input type="button" onClick="openCalendar(!   # "
  r << (y ? "this.form['#{name_prefix}year']," : "null,") <<
      (m ? "this.form['#{name_prefix}mon']," : "null,") <<
      (d ? "this.form['#{name_prefix}day']" : "null") << 
      %q!,event,'mm/dd');" value="...">!    # "
  return Amrita::SanitizedString[r]
end

# 同じ仕訳か？                                  
def same_entry?(x, y)
  return false if !x || !y
  if x.date == y.date && x.entry_no == y.entry_no
    true
  else 
    false
  end
end
