#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "pcview.rb"
require 'journal-data.rb'
require APP_ROOT + '/models/account_title'

class FSTemplate
  def initialize(user, begin_date, end_date)
    raise TypeError if !begin_date.is_a?(Date)
    raise TypeError if !end_date.is_a?(Date)

    @user = user

    op_date = user.user_setting.opening_date
    begin_date = op_date + 1 if begin_date <= op_date
    end_date = op_date + 1 if end_date <= op_date

    @account = Account.new(user)
    bal = @account.get_fs_multi([begin_date - 1, end_date])
    @bal_b = bal[0]
    @bal_e = bal[1]

    @total_b = [nil, 0, 0, 0, 0, 0, 0, 0]
    @bal_b.each {|code, amount|
      item = AccountTitle.find_item(code, @user)
      @total_b[item.ac_type] += amount * item.sign
    }
    @total = [nil, 0, 0, 0, 0, 0, 0, 0]
    @bal_e.each {|code, amount|
      item = AccountTitle.find_item(code, @user)
      @total[item.ac_type] += amount * item.sign
    }

    # B/Sの貸借差額を未処分利益に加算
    @unappropriated_item = user.user_setting.unappropriated_item
    item = AccountTitle.find_item(@unappropriated_item, @user)
    surplus_b = @total_b[1] - @total_b[2] - @total_b[3]
    surplus_e = @total[1] - @total[2] - @total[3]
    @bal_b[@unappropriated_item] =
                (@bal_b[@unappropriated_item] || 0) + surplus_b * item.sign
    @bal_e[@unappropriated_item] =
                (@bal_e[@unappropriated_item] || 0) + surplus_e * item.sign
    @total_b[3] += surplus_b
    @total[3] += surplus_e

    @ary2 = [nil, [], [], [], [], [], [], []]
    @bal_e.to_a.sort {|x, y| x[0] <=> y[0]}.each {|code, amount|
      item = AccountTitle.find_item(code, @user)
      if amount != 0 || item.ac_type <= 3 && @bal_b[code] != 0
        @ary2[item.ac_type] << [code, amount]
      end
    }

    @begin_date = begin_date
    @end_date = end_date
  end
  attr_reader :begin_date, :end_date
  
  # render結果を得る
  def bs()
    model = bs_body(@bal_b)
    return mod_render("bs.div.html", model)
  end
  
  # render結果を得る
  def pl()
    model = pl_body(@begin_date, @end_date)
    return mod_render("pl.div.html", model)
  end

  # render結果を得る
  def cf()
    model = cf_body(@bal_b, @bal_e)
    return mod_render("cf.div.html", model)
  end

  # render結果を得る
  def changes_in_equity()
    model = changes_in_equity_body(@begin_date, @end_date, @bal_b, @bal_e)
    return mod_render("eq.div.html", model)
  end
  
  private
  # balance sheet
  def bs_body(bal_b)
    model = {
      :prev_date => format_date(begin_date - 1),
      :bs_ed => format_date(end_date),
      :bs_total_assets1 => decimal_format(@total_b[1]),
      :bs_total_assets2 => decimal_format(@total[1] - @total_b[1]),
      :bs_total_assets3 => decimal_format(@total[1]),
      :bs_total_liabilities1 => decimal_format(@total_b[2]),
      :bs_total_liabilities2 => decimal_format(@total[2] - @total_b[2]),
      :bs_total_liabilities3 => decimal_format(@total[2]),
      :bs_total_equity1 => decimal_format(@total_b[3]),
      :bs_total_equity2 => decimal_format(@total[3] - @total_b[3]),
      :bs_total_equity3 => decimal_format(@total[3]),
      :bs_cr_total1 => decimal_format(@total_b[2] + @total_b[3]),
      :bs_cr_total2 => decimal_format(@total[2] + @total[3] - (@total_b[2] + @total_b[3])),
      :bs_cr_total3 => decimal_format(@total[2] + @total[3])
    }
    
    (1..3).each {|typ|
      node = [nil, "assets", "liabilities", "equity",
              "sales", "operating-costs", "other-income", "other-income"][typ]
      model[node.intern] = []
      @ary2[typ].each {|v|
        item = AccountTitle.find_item(v[0], @user)
        h = {
          :item => CGI.escapeHTML(item.name),
          :amount1 => decimal_format((bal_b[v[0]] || 0) * item.sign),
          :amount2 => decimal_format((v[1] - (bal_b[v[0]] || 0)) * item.sign),
          :amount3 => decimal_format(v[1] * item.sign)
        }
        model[node.intern] << h
      }
    }

    return model
  end

  # Income statement
  def pl_body(begin_date, end_date)
    model = {
      :b_date => format_date(begin_date),
      :e_date => format_date(end_date),
      :range => "金額",
      :net_sales => decimal_format(@total[4]),
      :op_costs_total => decimal_format(@total[5]),
      :operating_income => decimal_format(@total[4] - @total[5]),
      :other_total => decimal_format(@total[6] - @total[7]),
      :profit => decimal_format(@total[4] - @total[5] + @total[6] - @total[7])
    }

    (4..7).each {|typ|
      node = [nil, "assets", "liabilities", "equity",
              "sales", "operating_costs", "other_income", "other_income"][typ]
      raise "internal error" if !node

      model[node.intern] = []
      @ary2[typ].each {|v|
        item = AccountTitle.find_item(v[0], @user)
        line = {
          :item => item.name,
        }
        h = {
          :link => "ledger.rb?action=view&amp;byear=#{begin_date.year}&amp;" +
                   "bmon=#{begin_date.mon}&amp;bday=#{begin_date.day}&amp;" +
                   "eyear=#{end_date.year}&amp;emon=#{end_date.mon}&amp;" +
                   "eday=#{end_date.day}&amp;code=#{item.id}"
        }
        if typ == 7
          h[:amount] = decimal_format(v[1] * item.sign * -1)
        else
          h[:amount] = decimal_format(v[1] * item.sign)
        end
        line[:amounts] = h
        model[node.intern] << line
      }
    }

    return model
  end

  # Cash flow statement
  def cf_body(bal_b, bal_e)
    bsdiff = {}
    cash_b = 0
    cash_e = 0
    bal_b.each {|i, amount|
      item = AccountTitle.find_item(i, @user)
      cash_b += amount if item.cf_type == 4
      bsdiff[i] = amount - (bal_e[i] || 0) if item.bs?
    }
    bal_e.each {|i, amount|
      item = AccountTitle.find_item(i, @user)
      cash_e += amount if item.cf_type == 4
      if item.bs?
        bsdiff[i] = (bal_b[i] || 0) - amount
      else
        bsdiff[i] = -amount
      end
    }
    bsdiff_ary = bsdiff.to_a.sort {|x, y| x[0] <=> y[0]}
    
    model = {
      :b_date => begin_date,
      :e_date => end_date,
      :range => "金額",
      :cf_operating => [],
      :cf_investing => [],
      :cf_financing => []
    }

    # 営業C/F
    profit = o_cf = @total[4] - @total[5] + @total[6] - @total[7]
    bsdiff_ary.each {|code, diff|
      item = AccountTitle.find_item(code, @user)
      diff = diff - profit if code == @unappropriated_item
      if item.bs?
        if item.cf_type == 1 && diff != 0
          o_cf += diff
          model[:cf_operating] << {
            :item => CGI.escapeHTML(item.name) + " の増減",
            :amount => decimal_format(diff)
          }
        end
      else
        if item.cf_type != 1 && diff != 0
          o_cf -= diff
          model[:cf_operating] << {
            :item => CGI.escapeHTML(item.name),
            :amount => decimal_format(-diff)
          }
        end
      end
    }
    
    # 投資C/F
    i_cf = 0
    bsdiff_ary.each {|i, diff|
      item = AccountTitle.find_item(i, @user)
      if item.cf_type == 2 && diff != 0
        i_cf += diff
        model[:cf_investing] << {
          :item => item.name + (item.bs? ? " の増減" : ""),
          :amount => decimal_format(diff)
        }
      end
    }

    f_cf = 0
    bsdiff_ary.each {|i, diff|
      item = AccountTitle.find_item(i, @user)
      if item.cf_type == 3 && diff != 0
        f_cf += diff
        model[:cf_financing] << {
          :item => item.name + (item.bs? ? " の増減" : ""),
          :amount => decimal_format(diff)
        }
      end
    }

    model.update({
      :profit => decimal_format(profit),
      :cf_net_operating => decimal_format(o_cf),
      :cf_net_investing => decimal_format(i_cf),
      :cf_net_financing => decimal_format(f_cf),
      :cf_net_increase => decimal_format(o_cf + i_cf + f_cf),
      :cf_at_beginning => decimal_format(cash_b),
      :cf_at_end => decimal_format(cash_e)
    })
    return model
  end

  def index_of_ary(ary, code)
    ary.each_with_index {|e, i|
      return i if e.id == code
    }
    return nil
  end

  # Statement of changes in equity
  def changes_in_equity_body(begin_date, end_date, bal_b, bal_e)
    raise TypeError if !begin_date.is_a?(Date)
    raise TypeError if !end_date.is_a?(Date)

    col = AccountTitle.find(:all, 
                   :conditions => ['user_id = ? and ac_type = 3', @user.id],
                   :order => 'sort')
    sheet = []

    # 前期末
    row = []
    bal_b.each {|code, amount|
      idx = index_of_ary(col, code)
      row[idx + 2] = -amount if idx
    }
    row[0] = begin_date - 1
    sheet << row

    idx = index_of_ary(col, @unappropriated_item)
    sheet[0][idx + 2] = sheet[0][idx + 2].to_i + @total_b[1] - @total_b[2] - @total_b[3]

    # 期中の変動
    records = JournalEntry.find(:all, 
                      :conditions => ['user_id = ? and date between ? and ?',
                                      @user.id, begin_date, end_date],
                      :order => 'date')
    records.each {|r|
      row = []
      add = false
      idx = index_of_ary(col, r.dr_code)
      if idx
        row[idx + 2] = -r.amount
        row[0] = r.date
        row[1] = r.comment
        add = true
      end
      idx = index_of_ary(col, r.cr_code)
      if idx
        row[idx + 2] = +r.amount
        row[0] = r.date
        row[1] = r.comment
        add = true
      end
      sheet << row if add
    }

    idx = index_of_ary(col, @unappropriated_item)
    row = []
    row[0] = end_date
    row[1] = "当期純利益"
    row[idx ? idx + 2 : col.size + 1] = @total[4] - @total[5] + @total[6] - @total[7]
    sheet << row

    sheet.sort! {|x, y| x[0] <=> y[0]}

=begin
    # 残高も変動もない科目を削除
    col_r = []
    sheet_r = []
    sheet.each_with_index {|row, line| sheet_r[line] = [row.shift]}
    col.each_with_index {|item, idx|
      sheet.each {|row|
        if row[idx]
          col_r[col_r.size] = col[idx]
          sheet_r.each {|row_r|
            row_r[col_r.size] = row[idx]
          }
          break
        end
      }
    }
=end
    col_r = col; sheet_r = sheet; # DEBUG
    
    model = {
      :prev_date => format_date(begin_date - 1),
      :bs_ed => format_date(end_date),
      :ac_list => [],
      :s_list => [],
      :changes_list => [],
      :e_list => []
    }

    # 科目を表示
    col_r.each {|item| model[:ac_list] << item.name}

    # 前期末を表示
    (0 .. col_r.size - 1).each {|idx|
      val = sheet_r[0][idx + 2]
      model[:s_list] << (val ? decimal_format(val) : "-")
    }

    # 変動を表示
    sheet_r.each_with_index {|row, rowidx|
      next if rowidx == 0
      v_list = []
      (0 .. col_r.size - 1).each {|idx|
        val = row[idx + 2]
        v_list << (val ? decimal_format(val) : "")
      }
      model[:changes_list] << {
        :comment => CGI.escapeHTML(row[0].to_s + row[1].to_s),
        :v_list => v_list
      }
    }

    # 今期末を表示
    col_r.each_with_index {|item, idx|
      b = 0
      sheet_r.each {|row| b += row[idx + 2].to_i}
      model[:e_list] << (b != 0 ? decimal_format(b) : "-")
    }
    return model
  end
end

class ViewFS < Wafu::Action
  def execute(af)
    fs = FSTemplate.new(@app.user, af.begin_date, af.end_date)

    model = {
      :b_date => format_date_long(fs.begin_date),
      :e_date => format_date_long(fs.end_date),
      :bs_ed => format_date(fs.end_date),

      :journal_link => "jour2.rb?byear=#{fs.begin_date.year}&amp;" +
          "bmon=#{fs.begin_date.mon}&amp;bday=#{fs.begin_date.day}&amp;" +
          "eyear=#{fs.end_date.year}&amp;emon=#{fs.end_date.mon}&amp;" +
          "eday=#{fs.end_date.day}",
      :bs_mod => Amrita::SanitizedString[fs.bs()],
      :pl_mod => Amrita::SanitizedString[fs.pl()],
      :cf_mod => Amrita::SanitizedString[fs.cf()],
      :eq_mod => Amrita::SanitizedString[fs.changes_in_equity()]
    }

    @app.page_out("pl2.html", model)
  end
end

class InputRangeReload < Wafu::Action
  def execute(af)
    model = {
      :byear => af.by,
      :bmon => af.bm,
      :bday => af.bd,
      :eyear => af.ey,
      :emon => af.em,
      :eday => af.ed
    }
    if af.error?
      model[:error] = af.errors
    end
    @app.page_out("pl.html", model)
  end
end

class FSForm < PCView
  def main()
    actions = {
      "*" => [DateRangeActionForm, ViewFS, InputRangeReload]
    }
    dispatch(actions)
  end
end

if __FILE__ == $0
  FSForm.new.run
end
