#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 請求書 (AP) の編集

require "pcview.rb"
require APP_ROOT + '/models/invoice'
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/journal_entry'
require LIBRARY_DIR + '/datesup.rb'

class FirstCheck < Wafu::ActionForm
  def setup
    @invoice_id = @app.cgi.get('id').to_i
    if @invoice_id
      @inv = Invoice.find(:first, 
              :conditions => ['user_id = ? AND id = ?', @app.user.id, @invoice_id])
      if @inv
        @ary = @inv.journal_entries
        @lines = @ary.size + 3
        @year = @inv.date.year
        @mon = @inv.date.mon
        @day = @inv.date.day
        @partner = @inv.partner_id
      end
    end
  end

  def validate
    err = []
    err << "internal error: invoiceが指定されていません。" if !@inv
    return set_error(err)
  end
  attr_reader :invoice_id, :lines, :year, :mon, :day,
              :partner, :ary
end

class ErrorForm < Wafu::Action
  def execute(af)
    model = {}
    if af.error?
      model[:errors] = af.errors
    end
    @app.page_out("purchase_new_error.html", model)
    return nil
  end
end

class EditForm < Wafu::Action
  def execute(af)
    period = UserSetting.get_addable_period(@app.user)
    model = {
      :period => "#{period[0]} 〜 #{period[1]}",
      :list => [],
      :invoice_id => af.invoice_id,
      :lines => af.lines,
      :sel_date => get_sel_date("", af.year, af.mon, af.day),
      :pay_type => get_sel_partners(@app.user, "pay_type", af.partner.to_i),
      :notice => @app.get_flash("notice")
    }

    if af.error?
      model[:error] = {:message => af.errors}
    end

    (1..af.lines).each {|i|
      if af.ary[i - 1]
        h = {
          :accode => get_sel_purchase_items(@app.user, "accode-#{i}", 
                                          af.ary[i-1][:dr_code].to_i),
          :amountid => "amount#{i}",
          :av => af.ary[i-1][:amount],
          :comment => "comment-#{i}",
          :cv => af.ary[i-1][:comment]
        }
      else
        h = {
          :accode => get_sel_purchase_items(@app.user, "accode-#{i}", 0),
          :amountid => "amount#{i}",
          :comment => "comment-#{i}"
        }
      end
      model[:list] << h
    }
    @app.page_out("purchase-edit.html", model)
  end
end

class DoneAF < Wafu::ActionForm
  attr_reader :year, :mon, :day, 
              :invoice_id, :partner, :lines, :ary
 
  def validate
    err = []
    @lines = @app.cgi.get("lines").to_i
    @invoice_id = @app.cgi.get('id')
    @year = @app.cgi.get('year')
    @mon = @app.cgi.get("mon")
    @day = @app.cgi.get("day")

    period = UserSetting.get_addable_period(@app.user)
    if !Date.exist?(@year.to_i, @mon.to_i, @day.to_i)
      err << "日付が不正。" 
    else
      date = Date.new(@year.to_i, @mon.to_i, @day.to_i)
      if date > period[1]
        err << "登録可能期間より未来の日付です。メニューから締め処理をおこなってください。"
      end
    end

    @partner = @app.cgi.get("pay_type")
    amount_total = 0
    count = 0
    @ary = []
    (1..lines).each {|i|
      @ary << [@app.cgi.get("accode-#{i}"), 
              @app.cgi.get("amount#{i}"),
              @app.cgi.get("comment-#{i}") ]
      amount_total += @ary.last[1].to_i
      count += 1 if @ary.last[1].to_i != 0
    }
    err << "支払額がマイナスです。" if amount_total < 0
    err << "一件も明細がありません。" if count == 0
      
    return set_error(err)
  end
end

class DoneForm < Wafu::Action
  def execute(af)
    user_setting = @app.user.user_setting

    period = UserSetting.get_addable_period(@app.user)
    date_ = Date.new(af.year.to_i, af.mon.to_i, af.day.to_i)
    raise if date_ > period[1]
    entry_date = [date_, period[0]].max

    Invoice.transaction {
      entry_no = (user_setting.entry_no_counter += 1)
      user_setting.save!
      partner = Partner.find(af.partner)
      inv = Invoice.find(af.invoice_id)
      raise if inv.user != @app.user

      # 明細
      new_je = []
      af.ary.each {|line|
        if line[1].to_i != 0
          e = JournalEntry.new(:user => @app.user, 
                               :date => entry_date,
                               :entry_no => entry_no,
                               :dr_code => line[0],
                               :cr_code => user_setting.payable_item,
                               :amount => line[1],
                               :remarks => partner.name,
                               :comment => line[2],
                               :invoice => inv)
          new_je << e
        end
      }
      raise if new_je.size == 0

      if inv.journal_entries.first.date >= period[0]
        # 古い仕訳は削除していい
        JournalEntry.delete_all ['user_id = ? AND invoice_id = ?',
                                 @app.user.id, af.invoice_id]
      else
        # 赤伝を入れる？
        old_je = JournalEntry.find :all,
                         :conditions => ['user_id = ? AND invoice_id = ?', 
                                         @app.user.id, af.invoice_id]
        if ac_amount(old_je) != ac_amount(new_je)
          old_je.each {|e|
            red = JournalEntry.new(:user => @app.user,
                                   :date => period[0],
                                   :entry_no => e.entry_no,
                                   :dr_code => e.dr_code,
                                   :cr_code => e.cr_code,
                                   :amount => -e.amount,
                                   :remarks => e.remarks,
                                   :comment => "取消: " + e.comment)
            red.save!
            e.invoice = nil
            e.save!
          }
        else
          # コメントだけの変更なら、元の仕訳を修正してしまう。
          JournalEntry.delete_all ['user_id = ? AND invoice_id = ?',
                                   @app.user.id, af.invoice_id]
        end
      end

      # 更新
      inv.attributes = {:date => date_, 
                        :partner => partner,
                        :amount_total => 0}
 
      new_je.each {|e|
        e.save!
        inv.amount_total += e.amount
      }
      raise if inv.amount_total < 0
      inv.save!
    } # of transaction
    @app.set_flash("notice", "notice: 伝票を更新しました。")
    @app.set_redirect "login.rb"
  end

  private
  def ac_amount(t)
    result = {}
    t.each {|r|
      result[r.dr_code] = result[r.dr_code].to_i + r.amount
      result[r.cr_code] = result[r.cr_code].to_i - r.amount
    }
    return result
  end
end

class PurchaseApp < PCView
  def main
    actions = {
      '*' => [FirstCheck, EditForm, ErrorForm],
      'determine' => [DoneAF, DoneForm, EditForm]
    }
    dispatch(actions)
  end
end

PurchaseApp.new.run()
