#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 請求書 (AP) の登録

require "pcview.rb"
require APP_ROOT + '/models/invoice'
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/journal_entry'
require LIBRARY_DIR + '/datesup.rb'

class FirstCheck < Wafu::ActionForm
  def setup
    date = Time.now
    date = Date.new(date.year, date.mon, date.day) - 1 if date.hour < 3
    @mon = date.mon
    @day = date.day
    @lines = 3 # magic
    @ary = [[nil, "", ""], [nil, "", ""], [nil, "", ""]]
  end

  def validate
    err = []
    err << "未払金勘定の設定がありません。[取引先]画面で設定してください。" if !@app.user.user_setting.payable_item
    err << "取引先が一件もありません。[取引先]画面で登録してください。" if !Partner.find(:first)
    return set_error(err)
  end
  attr_reader :mon, :day, :partner, :lines, :ary
end

class ErrorForm < Wafu::Action
  def execute(af)
    model = {}
    if af.error?
      model[:errors] = af.errors
    end
    @app.page_out("purchase_new_error.html", model)
    return nil
  end
end

class EditForm < Wafu::Action
  def execute(af)
    period = UserSetting.get_addable_period(@app.user)
    model = {
      :period => "#{period[0]} 〜 #{period[1]}",
      :list => [],
      :lines => af.lines,
      :sel_date => get_sel_date("", nil, af.mon, af.day),
      :pay_type => get_sel_partners(@app.user, "pay_type", af.partner.to_i),
      :notice => @app.get_flash("notice")
    }

    if af.error?
      model[:error] = {:message => af.errors}
    end

    (1..af.lines).each {|i|
      h = {
        :accode => get_sel_purchase_items(@app.user, "accode-#{i}", 
                                          af.ary[i-1][0].to_i),
        :amountid => "amount#{i}",
        :av => af.ary[i-1][1],
        :comment => "comment-#{i}",
        :cv => af.ary[i-1][2]
      }
      model[:list] << h
    }
    @app.page_out("purchase.html", model)
  end
end

class DoneAF < Wafu::ActionForm
  attr_reader :mon, :day, :partner, :lines, :ary
 
  def validate
    err = []
    @lines = @app.cgi.get("lines").to_i
    @mon = @app.cgi.get("mon")
    @day = @app.cgi.get("day")

    period = UserSetting.get_addable_period(@app.user)
    date = get_full_day(@mon.to_i, @day.to_i, period[0])
    if !date
      err << "日付が不正。" 
    elsif date > period[1]
      err << "登録可能期間より未来の日付です。メニューから締め処理をおこなってください。"
    end

    @partner = @app.cgi.get("pay_type")
    amount_total = 0
    count = 0
    @ary = []
    (1..lines).each {|i|
      @ary << [@app.cgi.get("accode-#{i}"), 
              @app.cgi.get("amount#{i}"),
              @app.cgi.get("comment-#{i}") ]
      amount_total += @ary.last[1].to_i
      count += 1 if @ary.last[1].to_i != 0
    }
    err << "支払額がマイナスです。" if amount_total < 0
    err << "一件も明細がありません。" if count == 0
      
    return set_error(err)
  end
end

class DoneForm < Wafu::Action
  def execute(af)
    user_setting = @app.user.user_setting

    period = UserSetting.get_addable_period(@app.user)
    date = get_full_day(af.mon.to_i, af.day.to_i, period[0])
    raise if date > period[1]
    entry_date = date >= period[0] ? date : period[0]

    Invoice.transaction {
      entry_no = (user_setting.entry_no_counter += 1)
      user_setting.save!
      partner = Partner.find(af.partner)

      inv = Invoice.create(:user => @app.user, 
                           :date => date, 
                           :partner => partner,
                           :amount_total => 0)
      af.ary.each {|line|
        if line[1].to_i != 0
          e = JournalEntry.new(:user => @app.user, 
                               :date => date,
                               :entry_no => entry_no,
                               :dr_code => line[0],
                               :cr_code => user_setting.payable_item,
                               :amount => line[1],
                               :remarks => partner.name,
                               :comment => line[2],
                               :invoice => inv)
          e.save!
          inv.amount_total += e.amount
          inv.journal_entries << e
        end
      }
      raise if inv.amount_total < 0
      raise if inv.journal_entries.size == 0
      inv.save!
    } 
    @app.set_flash("notice", "notice: 伝票を登録しました。")
    @app.set_redirect "purchase.rb"
  end
end

class PurchaseApp < PCView
  def main
    actions = {
      '*' => [FirstCheck, EditForm, ErrorForm],
      'determine' => [DoneAF, DoneForm, EditForm]
    }
    dispatch(actions)
  end
end

PurchaseApp.new.run()
