#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require 'pcview.rb'
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/journal_entry'

class FirstCheck < Wafu::ActionForm
  def validate
    @item_id = @app.cgi.get('ac', nil)
    if !@item_id
      err = ["internal error: idが指定されていない"]
      return set_error(err)
    end

    item = AccountTitle.find_item(@item_id, @app.user)
    @name = item.name
    @name_short = item.name_short
    @sort = item.sort
    @ac_type = item.ac_type
    @cf_type = item.cf_type
    @desc = item.description
    @active = item.active
  end
  attr_reader :item_id, :name, :name_short, :sort, :ac_type, :cf_type,
              :desc, :active
end

class EditForm < Wafu::Action
  def get_sel_actype(name, default)
    return menu_control(name, 1..7, default) {|k| AccountTitle::ACTYPE_NAMES[k] }
  end

  def get_sel_cftype(name, default)
    return menu_control(name, 1..4, default) {|k| AccountTitle::CFTYPE_NAMES[k]}
  end

  def has_journal?(item_id)
    j = JournalEntry.find(:first,
               :conditions => ['user_id = ? and (dr_code = ? or cr_code = ?)',
                               @app.user.id, item_id, item_id])
    return j ? true : false
  end

  def execute(af)
    item = AccountTitle.find_item(af.item_id, @app.user)

    model = {
      :id => af.item_id,
      :name => af.name,
      :mobname => af.name_short,
      :sort => af.sort,
      :ac_type => get_sel_actype("actype", af.ac_type),
      :cf_type => get_sel_cftype("cftype", af.cf_type),
      :desc => af.desc,
    }
    if has_journal?(af.item_id)
      model[:sel_active] = {:active => af.active != 0 ? nil: "checked"}
    else
      model[:delete] = true
    end
    @app.page_out('titles-edit.html', model)
  end
end

class UpdateAF < Wafu::ActionForm
  def validate
    n = {'id' => 'item_id', 
      'name' => 'name', 'mobname' => 'name_short', 'sort' => 'sort',
      'actype' => 'ac_type', 'cftype' => 'cf_type', 'desc' => 'desc',
      'active' => 'active'}
    n.each {|cginame, vname|
      instance_variable_set('@' + vname, @app.cgi.get(cginame, nil))
    }

    err = []
    if !@item_id
      err << 'internal error: idが指定されていない'
      return set_error(err)
    end
  end
  attr_reader :item_id, :name, :name_short, :sort, :ac_type, :cf_type,
              :desc, :active
end

class UpdateDone < Wafu::Action
  def execute(af)
    r = AccountTitle.find_item(af.item_id, @app.user)
    r.name, r.name_short, r.sort, r.ac_type, r.cf_type, r.description = 
      [af.name, af.name_short, af.sort, af.ac_type, af.cf_type, af.desc]
    r.active = (af.active && af.active != "") ? 0 : 1 # 画面はinactiveを選ぶ
    r.save!

    @app.set_flash "notice", "科目「#{af.name}」を更新しました。"
    @app.set_redirect "titles.rb"
  end
end

class DeleteDone < Wafu::Action
  def execute(af)
    AccountTitle.delete(af.item_id)

    @app.set_flash "notice", "科目「#{af.name}」を削除しました。"
    @app.set_redirect "titles.rb"
  end
end

class EditApp < PCView
  def main
    actions = {
      "*" => [FirstCheck, EditForm, nil],
      "update" => [UpdateAF, UpdateDone, EditForm],
      "delete" => [UpdateAF, DeleteDone, EditForm],
    }
    dispatch(actions)
  end
end

EditApp.new.run()
