#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 週次比較表

require "pcview.rb"
require "comparison.rb"

def get_sel_wday(name, default)
    menu_control(name, 0..6, default) {|i|
               %w(日曜日 月曜日 火曜日 水曜日 木曜日 金曜日 土曜日)[i]}
end

class WeekPageStart < Wafu::Action
  def execute(af)
    today = Date.today
    wday = @app.user.user_setting.week_last_wday || 0
    
    # 表示
    model = {
      :sel_wday => get_sel_wday("wday", wday),
      :ey => today.year,
      :em => today.mon,
      :ed => today.day
    }
    @app.page_out("week.html", model)
  end
end

class InputRangeReload < Wafu::Action
  def execute(af)
    model = {
      :sel_wday => get_sel_wday("wday", af.wday),
      :ey => af.ey,
      :em => af.em,
      :ed => af.ed
    }
    if af.error?
      model[:error] = {:message => af.errors}
    end
    @app.page_out("week.html", model)
  end
end

class WeekAF < Wafu::ActionForm
  attr_reader :ey, :em, :ed, :wday

  def setup()
    @ey = @app.cgi.get("ey")
    @em = @app.cgi.get("em")
    @ed = @app.cgi.get("ed")
    @wday = @app.cgi.get("wday").to_i
  end

  def validate()
    err = []
    if !Date.exist?(@ey.to_i, @em.to_i, @ed.to_i)
      err << "日付が正しくありません。"
    end
    return set_error(err)
  end
end

class WeekOut < Wafu::Action
  include ComparativeChart

  def get_prev_end(last_end, cur_end)
    raise TypeError if !cur_end.is_a?(Date) && !cur_end.is_a?(NilClass)

    if !cur_end
      return last_end
    else
      return cur_end - 7
    end
  end

  def get_last_end(af)
    end_date = Date.new(af.ey.to_i, af.em.to_i, af.ed.to_i)
    while end_date.wday != af.wday
      end_date += 1
    end
    @app.user.user_setting.week_last_wday = af.wday
    @app.user.user_setting.save!

    return end_date
  end

  def execute(af)
    begin
      table_out("週次推移表", get_last_end(af))
    rescue RangeError => e
      af.set_error([e.message])
      return InputRangeReload
    end
  end
end

class Form < PCView
  def main()
    actions = {
      "do" => [WeekAF, WeekOut, InputRangeReload],
      "*" => [nil, WeekPageStart, nil]
    }
    dispatch(actions)
  end
end

if __FILE__ == $0
  run(Form.new)
end
