#!/home/netsphere/bin/ruby
# -*- mode:ruby; coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "pcview.rb"
require "closing-common.rb"
require "conf/item-default.rb"
require APP_ROOT + '/models/account_title'

class Form < PCView
  def get_initial_titles()
    ary = Array.new
    DEFAULT_ITEMS.each {|e|
      ary << AccountTitle.new(
          :sort => e[0], :name => e[1], :name_short => e[2], 
          :ac_type => e[3], :cf_type => e[4], :description => e[5])
    }
    return ary
  end
  
  def add_line(i, item)
    line = {
      :code => "code-#{i}",
      :sort => "sort-#{i}",
      :name => "name-#{i}",
      :ns => "ns-#{i}",
      :actype => "actype-#{i}",
      :cftype => menu_control("cftype-#{i}", 1..4, 
                              item ? item.cf_type : nil) {|k|
        AccountTitle::CFTYPE_NAMES[k]},
      :desc => "desc-#{i}",
      :cv => (item ? item.id : nil),
      :sortv => (item ? item.sort : nil),
      :nv => (item ? item.name : nil),
      :sv => (item ? item.name_short : nil),
      :tv => (item ? item.ac_type : nil),
      :dv => (item ? item.description : nil)
    }
    return line
  end

  def group_desc(group)
    case group
    when 1
      "資産は，現金・預金などの財産や，未収入金など将来の入金となるものです。現金や通常使う預金はC/F区分を現金同等物、定期預金などは投資活動にします。"
    when 2
      "負債は，ローン，クレジットカードの残高など，将来何らかの支出（返済）が必要なものです。"
    when 3
      "純資産は，資産から負債を差し引いた，純粋な自身の財産です。"
    when 4
      "営業収益は，通常の給料、家賃収入など，経常的な収入です。"
    when 5
      "営業費用は，経常的に発生する支出です。金融費用、税金、異常な損失は含みません。"
    when 6
      "その他の収益は，金融収益（利息など），臨時的な収入です。"
    when 7
      "その他の費用は，金融費用，税金，臨時的な支出です。"
    end
  end

  def form_out(cgi)
    un_id = user.user_setting.unappropriated_item
    if un_id 
      # 未処分利益は表示しない。
      initlist = AccountTitle.find(:all, 
                                   :conditions => ['user_id = ? and id <> ?', 
                                                   user.id, un_id])
    else
      # 初回
      initlist = get_initial_titles()
    end

    model = {:table => []}
    i = 1
    (1..7).each {|group|  
      h = {
        :group => "#{group}. " + AccountTitle::ACTYPE_NAMES[group],
        :group_desc => group_desc(group),
        :aclist => []
      }
      initlist.each {|item|
        if item.ac_type == group
          h[:aclist] << add_line(i, item)
          i += 1
        end
      }
      (0..2).each {  # 新しい科目を追加できるように。
        h[:aclist] << add_line(i, nil)
        i += 1
      }
      model[:table] << h
    }
    model[:lines] = i

    page_out("titles.html", model)
  end

  def main
    if cgi.get("action", "") != "prev"
      # 月末日の設定
      set_closing_day(cgi, user)
    end

    # 勘定科目の設定の画面
    form_out(cgi)
  end
end

run(Form.new)
