#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "pcview.rb"
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/journal_entry'

class OpeningBalancePage < Wafu::Action
  # 勘定科目のデータベースへの登録
  def set_ac_titles(cgi)
    lines = cgi["lines"].first.to_i
    AccountTitle.transaction {
      (1..lines).each {|i|
        code = cgi["code-#{i}"].first.to_i
        sort = cgi["sort-#{i}"].first.to_i
        name = cgi["name-#{i}"].first
        name_short = cgi["ns-#{i}"].first
        ac_type = cgi["actype-#{i}"].first.to_i
        cf_type = cgi["cftype-#{i}"].first
        desc = cgi["desc-#{i}"].first
        if sort != 0 && (name && name.strip != "") && ac_type > 0
          if code != 0
            t = AccountTitle.find(:first, 
                                  :conditions => ["id = ? and user_id = ?",
                                                  code, @app.user.id])
          end
          t = AccountTitle.new(:user => @app.user) if !t
          t.sort = sort
          t.name = name.strip
          t.name_short = name_short.strip
          t.ac_type = ac_type
          t.cf_type = cf_type
          t.description = desc.strip
          t.active = 1
          t.save!
        end
      }
      # 未処分利益の登録
      if !@app.user.user_setting.unappropriated_item
        un = AccountTitle.new(:user => @app.user, 
                              :sort => 7330, 
                              :name => "未処分利益",
                              :name_short => "", 
                              :ac_type => 3, 
                              :cf_type => 1,
                              :description => "利益の繰越",
                              :active => 1)
        un.save!
        @app.user.user_setting.unappropriated_item = un.id
        @app.user.user_setting.save!
      end
    } # of transaction
  end
  
  def exe_start()
    set_ac_titles(@app.cgi)
    
    # 次のページの表示
    op_date = Date.today - 1
    model = {
      :yv => op_date.year,
      :mv => op_date.mon,
      :dv => op_date.day,
    }
    return model
  end

  def exe_reload(af)
    model = {
      :yv => af.year,
      :mv => af.mon,
      :dv => af.day
    }
    if af.error?
      model[:errors] = af.errors
    end
    return model
  end

  def execute(af)
    if af
      model = exe_reload(af)
    else
      model = exe_start()
    end

    un_id = @app.user.user_setting.unappropriated_item

    model[:bal_list] = []
    lines = 0
    (1..3).each {|typ|
      ac_titles = AccountTitle.find(:all, 
                   :conditions => ['user_id = ? and ac_type = ? and id <> ?', 
                                   @app.user.id, typ, un_id],
                   :order => "sort")
      if ac_titles.size > 0
        model_hash = {
          :ac_type => AccountTitle::ACTYPE_NAMES[typ] + "の部",
          :list => []
        }
        ac_titles.each {|e|
          lines += 1
          model_hash[:list] << {
            :ac_name => CGI.escapeHTML(e.name),
            :valname => "amount-#{lines}",
            :amount => af ? af.amounts[e.id] : "",
            :codename => "code-#{lines}",
            :codeval => e.id
          }
        }
        model[:bal_list] << model_hash
      end
    }
    model[:lines] = lines

    @app.page_out("welcome-opening-balance.html", model)
  end
end

class OpeningBalanceAF < Wafu::ActionForm
  attr_reader :year, :mon, :day
  attr_reader :amounts
  
  def setup()
    @year = @app.cgi.get("y")
    @mon = @app.cgi.get("m")
    @day = @app.cgi.get("d")
  end

  def validate()
    ret = []

    if !Date.exist?(@year.to_i, @mon.to_i, @day.to_i)
      ret << "日付が正しくありません。"
    elsif Date.new(@year.to_i, @mon.to_i, @day.to_i) >= Date.today
      ret << "日付は昨日以前にしてください。"
    end

    @amounts = {}
    lines = @app.cgi.get("lines").to_i
    (1..lines).each {|i|
      code = @app.cgi.get("code-#{i}").to_i
      amount = @app.cgi.get("amount-#{i}").to_i
      @amounts[code] = amount if code != 0
    }

    return set_error(ret)
  end
end

class SetDone < Wafu::Action
  def set_opening_balance(af)
    user_setting = @app.user.user_setting
    entry_no = (user_setting.entry_no_counter += 1)
    day = Date.new(af.year.to_i, af.mon.to_i, af.day.to_i)
    JournalEntry.transaction {
      af.amounts.each {|code, amount|
        if amount != 0
          item = AccountTitle.find_item(code, @app.user)
          r = JournalEntry.new(:user => @app.user, 
                           :date => day,
                           :entry_no => entry_no,
                           :dr_code => code,
                           :cr_code => user_setting.unappropriated_item,
                           :amount => amount * item.sign,
                           :remarks => "",
                           :comment => "初期設定")
          r.save!
        end
      }
    }
    user_setting.opening_date = user_setting.fixed_date = day
    user_setting.save!
  end

  def execute(af)
    set_opening_balance(af)
    @app.page_out("welcome-done.html", {})
  end
end

class Form < PCView
  def main()
    actions = {
      "*" => [nil, OpeningBalancePage, nil],
      "set" => [OpeningBalanceAF, SetDone, OpeningBalancePage]
    }
    dispatch(actions)
  end
end

run(Form.new)
